package dev.peerat.parser.java.operation;

import java.util.List;
import java.util.function.Function;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Operation;
import dev.peerat.parser.java.value.Value;

public class MethodCallOperation extends Operation{
	
	private Value start;
	private MethodCallOperation previous;
	private Token path;
	private List<Value> parameters;
	
	public MethodCallOperation(Value start, Token path, List<Value> parameters){
		this.start = start;
		this.path = path;
		this.parameters = parameters;
	}
	
	public MethodCallOperation(MethodCallOperation previous, Token path, List<Value> parameters){
		this((Value)null, path, parameters);
		this.previous = previous;
	}
	
	public MethodCallOperation getPrevious(){
		return this.previous;
	}
	
	public Token getPath(){
		return path;
	}
	
	public List<Value> getParameters(){
		return this.parameters;
	}
	
	@Override
	public void build(Builder builder) throws Exception{
		if(previous != null) previous.build(builder);
		//todo
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder) {
		if(start != null && finder.apply(start)) return (E)start;
		if(previous != null) if(finder.apply(previous)) return (E) previous;
		if(this.parameters != null){
			for(Value param : this.parameters){
				if(finder.apply(param)) return (E)param;
			}
		}
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
		if(start != null){
			if(finder.apply(start)) list.add((E)start);
			start.findAll(finder, list);
		}
		if(previous != null){
			if(finder.apply(previous)) list.add((E) previous);
			previous.findAll(finder, list);
		}
		if(this.parameters != null){
			for(Value param : this.parameters){
				if(finder.apply(param)) list.add((E)param);
				param.findAll(finder, list);
			}
		}
	}
}
